USE omop4;
GO
/* *** BLOCK 1: Extract relevant data element FROM source data and store in #tmp *** */
IF OBJECT_ID('tempdb..#tmp') IS NOT NULL
	DROP TABLE #tmp;
CREATE TABLE #tmp (
	person_id INT
	,eventDate DATE
	,eventConceptId VARCHAR(100)
	,eventType VARCHAR(100)
	,eventNumValue FLOAT NULL
	,eventStringValue VARCHAR(100) NULL
	);

INSERT INTO #tmp
SELECT DISTINCT *
FROM (
	SELECT P0.person_id
		,U.eventDate
		,U.eventConceptId
		,U.eventType
		,U.eventNumValue
		,U.eventStringValue
	FROM omop4.nis7022.eMergeCohort P0
	LEFT JOIN (
	/* CONTACT may also contain REPORT and NOTE */
		 SELECT person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'Contact' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IS NOT NULL
		AND DATEDIFF(year, condition_start_date,GETDATE()) <= 5
		AND DATEDIFF(year, condition_start_date,GETDATE()) >=0
	
	UNION ALL
	SELECT person_id
	      ,visit_start_date AS eventDate
	      , NULL AS eventConceptId
	      ,'Contact' AS eventType
	      ,NULL AS eventNumValue
		    ,NULL AS eventStringValue
	  FROM omop4.dbo.visit_occurrence
	  WHERE visit_start_date IS NOT NULL
	  AND DATEDIFF(year, visit_start_date,GETDATE()) <= 5
	  AND DATEDIFF(year, visit_start_date,GETDATE()) >=0
	  
	  UNION ALL
	  SELECT person_id
			,drug_exposure_start_date AS eventDate
			,drug_concept_id AS eventConceptId
			,'Contact' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.drug_exposure
		WHERE drug_exposure_start_date IS NOT NULL
		AND DATEDIFF(year, drug_exposure_start_date,GETDATE()) <= 5
	    AND DATEDIFF(year, drug_exposure_start_date,GETDATE()) >=0
		
		UNION ALL
		SELECT person_id
			,observation_date AS eventDate
			,observation_concept_id AS eventConceptId
			,'Contact' AS eventType
			,value_as_number AS eventNumValue
		    ,value_as_string AS eventStringValue
		FROM omop4.dbo.observation
		WHERE observation_date IS NOT NULL
		AND DATEDIFF(year, observation_date,GETDATE()) <= 5
	    AND DATEDIFF(year, observation_date,GETDATE()) >=0
		
		UNION ALL
		SELECT person_id
			,procedure_date AS eventDate
			,procedure_concept_id AS eventConceptId
			,'Contact' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.procedure_occurrence
		WHERE procedure_concept_id IS NOT NULL
		AND DATEDIFF(year, procedure_date,GETDATE()) <= 5
	    AND DATEDIFF(year, procedure_date,GETDATE()) >=0
	
		UNION ALL
    	SELECT person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'DxCaad' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IN ('21013914','4288310','36718038','442615','4043731','36718001')
		
		UNION ALL
		SELECT person_id
			,procedure_date AS eventDate
			,procedure_concept_id AS eventConceptId
			,'ProcEndarterectomy' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.procedure_occurrence
		WHERE procedure_concept_id in ('2107740','2107777','2000061','2002027')
		
		UNION ALL
		SELECT person_id
			,procedure_date AS eventDate
			,procedure_concept_id AS eventConceptId
			,'ProcCtrlExcl' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.procedure_occurrence P
		WHERE procedure_concept_id in ('2100955','2100954','2107453','2107740','2107777','2107836','2000059','2000061','2002016','2002027','2002111')
		
		UNION ALL
		SELECT person_id
			,observation_date AS eventDate
			,observation_concept_id AS eventConceptId
			,'Lab' AS eventType
			,value_as_number AS eventNumValue
		    ,value_as_string AS eventStringValue
		FROM omop4.dbo.observation
		WHERE (observation_concept_id in ('3031776','3002651','3005104','3015232','3033364','3049873','40760031','3001547','3035774','3035660','3036009','3043623','3020324','3035135','40762131','3027114','3019053')
					AND (value_as_number is not NULL AND value_as_number >400) 
					)

		) U ON P0.person_id = U.person_id
	) Utotal;

/* *** BLOCK 2: Get algorithm related variables *** */
IF OBJECT_ID('tempdb..#AlgVar') IS NOT NULL
	DROP TABLE #AlgVar;

CREATE TABLE #AlgVar (
	person_id INT NOT NULL
	,diseaseName VARCHAR(50) NOT NULL
	,caseControl VARCHAR(50) NOT NULL
	,C00 INT NOT NULL
	,C01 INT NOT NULL
	,C02 INT NOT NULL
	,C03 INT NOT NULL
	,C04 INT NOT NULL
	,C05 INT NOT NULL
	,C06 INT NOT NULL
	,C07 INT NOT NULL
	);

INSERT INTO #AlgVar
SELECT DISTINCT J.person_id
	,'Temp' AS diseaseName
	,'CASE/CONTROL' AS caseControl
	,ISNULL(J0.dxCaadCnt, 0) AS C00
	,ISNULL(J1.contactFreq1, 0) AS C01
	,ISNULL(J2.contactFreq2, 0) AS C02
	,ISNULL(J3.contactFreq3, 0) AS C03
	,ISNULL(J4.labCnt, 0) AS C04
	,ISNULL(J5.procEndarterectomyCnt, 0) AS C05
	,ISNULL(J6.procCtrlExclCnt, 0) AS C06
	,ISNULL(J7.dxCaad30ApartCnt, 0) AS C07
FROM omop4.nis7022.eMergeCohort J

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS dxCaadCnt
	FROM #tmp
	WHERE eventType = 'DxCaad'
	GROUP BY person_id
	) J0 
ON J.person_id = J0.person_id

LEFT JOIN (
/* 6.1.1 */
	SELECT person_id, COUNT(dateHavingContact) AS contactFreq1
	FROM (
		 SELECT person_id
	,DATEPART(YEAR, eventDate) AS dateHavingContact
		,count(distinct eventDate) AS cnt
	FROM #tmp
	WHERE eventType = 'Contact'
	GROUP BY person_id, DATEPART(YEAR, eventDate)
) U
 where U.cnt >=1
 GROUP BY person_id
	) J1
ON J.person_id = J1.person_id

LEFT JOIN (
/* 6.1.2 */
	SELECT person_id, COUNT(dateHavingContact) AS contactFreq2
	FROM (
		 SELECT person_id
	,CONCAT(year(eventDate),'_',DATEPART(QUARTER, eventDate)) AS dateHavingContact
		,count(distinct eventDate) AS cnt
	FROM #tmp
	WHERE eventType = 'Contact'
	GROUP BY person_id, CONCAT(year(eventDate),'_',DATEPART(QUARTER, eventDate))
) U
 WHERE U.cnt >=1
 GROUP BY person_id
	) J2 
ON J.person_id = J2.person_id

LEFT JOIN (
/* 6.1.3 */
	SELECT person_id, COUNT(dateHavingContact) AS contactFreq3
	FROM (
		 SELECT person_id
	,CONCAT(year(eventDate),'_',DATEPART(MONTH, eventDate)) AS dateHavingContact
		,count(distinct eventDate) AS cnt
	FROM #tmp
	WHERE eventType = 'Contact'
	GROUP BY person_id, CONCAT(year(eventDate),'_',DATEPART(MONTH, eventDate))
) U
 WHERE U.cnt >=1
 GROUP BY person_id
	) J3 
ON J.person_id = J3.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS labCnt
	FROM #tmp
	WHERE eventType = 'Lab'
	
	group by person_id
	) J4 
ON J.person_id = J4.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS procEndarterectomyCnt
	FROM #tmp
	WHERE eventType = 'ProcEndarterectomy'
	
	group by person_id
	) J5
ON J.person_id = J5.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS procCtrlExclCnt
	FROM #tmp
	WHERE eventType = 'ProcCtrlExcl'
	
	group by person_id
	) J6
ON J.person_id = J6.person_id

/* 7.5 */
LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS dxCaad30ApartCnt
	FROM #tmp T
	WHERE eventType = 'DxCaad'
	AND DATEADD(day, 30, eventDate) <= (SELECT MAX(eventDate) as maxDate
	FROM #tmp T1
	WHERE eventType = 'DxCaad' AND T1.person_id = T.person_id )
	GROUP BY person_id
	) J7 
ON J.person_id = J7.person_id

;



/* *** BLOCK 3: Phenotyping **** */
IF OBJECT_ID('tempdb..#phenotype') IS NOT NULL
	DROP TABLE #phenotype;

CREATE TABLE #phenotype (
	person_id INT
	,pCase VARCHAR(50)
	,pCtrl VARCHAR(50)
	);

INSERT INTO #phenotype
SELECT DISTINCT CL.person_id /* CL has duplicate person_id */
	,ISNULL(U1.phenotype, 'NO') AS pCase
	,ISNULL(U2.phenotype, 'NO') AS pCtrl
FROM omop4.nis7022.eMergeCohort CL
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			((C01 >= 2 OR
			C02 >= 3 OR 
			C03 >= 4) AND
			C04 = 0 AND
			C05 >= 1)
			
			OR
			
			((C01 >= 2 OR
			C02 >= 3 OR 
			C03 >= 4) AND
			C04 = 0 AND
			C07 >= 2)
			)
	) U1 ON CL.person_id = U1.person_id
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			(C01 >= 2 OR
			C02 >= 3 OR 
			C03 >= 4) AND
			C06 = 0 AND
			C00 = 0
			)
	) U2 ON CL.person_id = U2.person_id;

/* *** BLOCK 4: Write each patient phenotyping result (PR) to a table *** */
IF COL_LENGTH('omop4.nis7022.eMergePR', 'CAADcase') IS NULL
	ALTER TABLE omop4.nis7022.eMergePR ADD CAADcase VARCHAR(50);

IF COL_LENGTH('omop4.nis7022.eMergePR', 'CAADctrl') IS NULL
	ALTER TABLE omop4.nis7022.eMergePR ADD CAADctrl VARCHAR(50);

UPDATE omop4.nis7022.eMergePR
SET CAADcase = (
		SELECT pCase
		FROM #phenotype
		WHERE #phenotype.person_id = omop4.nis7022.eMergePR.person_id
		)
	,CAADctrl = (
		SELECT pCtrl
		FROM #phenotype
		WHERE #phenotype.person_id = omop4.nis7022.eMergePR.person_id
		);

/* *** BLOCK 5: Get count for each phenotype *** */
SELECT U1.CaseCnt
	,U2.CtrlCnt
FROM (
	SELECT Count(DISTINCT person_id) AS CaseCnt
	FROM #phenotype
	WHERE pCase = 'YES'
	) U1
	,(
		SELECT Count(DISTINCT person_id) AS CtrlCnt
		FROM #phenotype
		WHERE pCtrl = 'YES'
		) U2;

/* *** For phekb submission *** */
SELECT person_id AS eMERGE_ID
	,CASE WHEN pCase = 'YES' THEN 'C49152'
		WHEN pCtrl = 'YES' THEN 'C28143'
		WHEN pCase = 'NO' AND pCtrl = 'NO' THEN 'NO' END AS Case_Control
FROM #phenotype
ORDER BY eMERGE_ID
;